@@ -8,7 +8,7 @@ import warnings
 from functools import lru_cache
 from string import Formatter
 from types import CodeType
-from typing import Any, Final, Literal
+from typing import Any, Literal
 
 import executing
 from typing_extensions import NotRequired, TypedDict
@@ -33,8 +33,6 @@ class ArgChunk(TypedDict):
 
 
 class ChunksFormatter(Formatter):
-    NONE_REPR: Final[str] = 'null'
-
     def chunks(
         self,
         format_string: str,
@@ -95,15 +93,12 @@ class ChunksFormatter(Formatter):
                 return None
 
             msg = '`executing` failed to find a node.'
-            if sys.version_info[:2] < (3, 11):
-                # inspect_arguments is only on be default for 3.11+ for this reason.
-                # The AST modifications made by auto-tracing and @instrument
+            if sys.version_info[:2] < (3, 11):  # pragma: no cover
+                # inspect_arguments is only on by default for 3.11+ for this reason.
+                # The AST modifications made by auto-tracing
                 # mean that the bytecode doesn't match the source code seen by `executing`.
                 # In 3.11+, a different algorithm is used by `executing` which can deal with this.
-                msg += (
-                    ' This may be caused by a combination of using Python < 3.11 '
-                    'and auto-tracing or @logfire.instrument.'
-                )
+                msg += ' This may be caused by a combination of using Python < 3.11 and auto-tracing.'
 
             # Try a simple fallback heuristic to find the node which should work in most cases.
             main_nodes: list[ast.AST] = []
@@ -284,15 +279,12 @@ class ChunksFormatter(Formatter):
                 )
                 format_spec = ''.join(chunk['v'] for chunk in format_spec_chunks)
 
-                if obj is None:
-                    value = self.NONE_REPR
-                else:
-                    try:
-                        value = self.format_field(obj, format_spec)
-                    except Exception as exc:
-                        raise KnownFormattingError(f'Error formatting field {{{field_name}}}: {exc}') from exc
-                    value, value_scrubbed = self._clean_value(field_name, value, scrubber)
-                    scrubbed += value_scrubbed
+                try:
+                    value = self.format_field(obj, format_spec)
+                except Exception as exc:
+                    raise KnownFormattingError(f'Error formatting field {{{field_name}}}: {exc}') from exc
+                value, value_scrubbed = self._clean_value(field_name, value, scrubber)
+                scrubbed += value_scrubbed
                 d: ArgChunk = {'v': value, 't': 'arg'}
                 if format_spec:
                     d['spec'] = format_spec
