@@ -8,7 +8,7 @@ from unittest import mock
 import pytest
 from dirty_equals import IsJson
 from fastapi import BackgroundTasks, FastAPI, Response, WebSocket
-from fastapi.exceptions import RequestValidationError
+from fastapi.exceptions import HTTPException, RequestValidationError
 from fastapi.params import Header
 from fastapi.security import SecurityScopes
 from fastapi.staticfiles import StaticFiles
@@ -23,6 +23,8 @@ import logfire
 import logfire._internal
 import logfire._internal.integrations
 import logfire._internal.integrations.fastapi
+from logfire._internal.constants import LEVEL_NUMBERS
+from logfire._internal.main import set_user_attributes_on_raw_span
 from logfire.testing import TestExporter
 
 
@@ -73,6 +75,10 @@ async def echo_body(request: Request):
     return await request.body()
 
 
+async def bad_request_error():
+    raise HTTPException(400)
+
+
 async def websocket_endpoint(websocket: WebSocket, name: str):
     logfire.info('websocket_endpoint: {name}', name=name)
     await websocket.accept()
@@ -96,6 +102,7 @@ def app():
     app.get('/other', name='other_route_name', operation_id='other_route_operation_id')(other_route)
     app.get('/exception')(exception)
     app.get('/validation_error')(validation_error)
+    app.get('/bad_request_error')(bad_request_error)
     app.get('/with_path_param/{param}')(with_path_param)
     app.get('/secret/{path_param}', name='secret')(get_secret)
     app.websocket('/ws/{name}')(websocket_endpoint)
@@ -191,6 +198,14 @@ def test_404(client: TestClient, exporter: TestExporter) -> None:
     )
 
 
+def test_400(client: TestClient, exporter: TestExporter) -> None:
+    response = client.get('/bad_request_error')
+    assert response.status_code == 400
+
+    [span] = [span for span in exporter.exported_spans if span.events]
+    assert span.attributes and span.attributes['logfire.level_num'] == LEVEL_NUMBERS['warn']
+
+
 def test_path_param(client: TestClient, exporter: TestExporter) -> None:
     response = client.get('/with_path_param/param_val')
     assert response.status_code == 200
@@ -201,7 +216,7 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
     span_dicts = exporter.exported_spans_as_dict(_include_pending_spans=True)
 
     # Highlights of the below mega-assert:
-    assert span_dicts[0]['name'] == 'GET /with_path_param/{param} (pending)'
+    assert span_dicts[0]['name'] == 'GET /with_path_param/{param}'
     assert span_dicts[0]['attributes']['logfire.msg'] == 'GET /with_path_param/param_val'
     assert span_dicts[-1]['name'] == 'GET /with_path_param/{param}'
     assert span_dicts[-1]['attributes']['logfire.msg'] == 'GET /with_path_param/param_val'
@@ -211,7 +226,7 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
     assert span_dicts == snapshot(
         [
             {
-                'name': 'GET /with_path_param/{param} (pending)',
+                'name': 'GET /with_path_param/{param}',
                 'context': {'trace_id': 1, 'span_id': 2, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'start_time': 1000000000,
@@ -243,7 +258,7 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
                 },
             },
             {
-                'name': 'FastAPI arguments (pending)',
+                'name': 'FastAPI arguments',
                 'context': {'trace_id': 1, 'span_id': 4, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'start_time': 2000000000,
@@ -265,11 +280,16 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
                     'logfire.msg_template': 'FastAPI arguments',
                     'logfire.msg': 'FastAPI arguments',
                     'logfire.span_type': 'span',
+                    'http.method': 'GET',
+                    'fastapi.route.name': 'with_path_param',
+                    'http.route': '/with_path_param/{param}',
+                    'fastapi.route.operation_id': 'null',
                     'logfire.level_num': 5,
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                 },
             },
             {
-                'name': '{method} {http.route} ({code.function}) (pending)',
+                'name': '{method} {http.route} ({code.function})',
                 'context': {'trace_id': 1, 'span_id': 6, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 5, 'is_remote': False},
                 'start_time': 4000000000,
@@ -308,7 +328,7 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
                 },
             },
             {
-                'name': 'GET /with_path_param/{param} http send (pending)',
+                'name': 'GET /with_path_param/{param} http send',
                 'context': {'trace_id': 1, 'span_id': 8, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 7, 'is_remote': False},
                 'start_time': 6000000000,
@@ -336,7 +356,7 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
                 },
             },
             {
-                'name': 'GET /with_path_param/{param} http send (pending)',
+                'name': 'GET /with_path_param/{param} http send',
                 'context': {'trace_id': 1, 'span_id': 10, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 9, 'is_remote': False},
                 'start_time': 8000000000,
@@ -390,6 +410,9 @@ def test_path_param(client: TestClient, exporter: TestExporter) -> None:
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/with_path_param/{param}',
+                    'fastapi.route.name': 'with_path_param',
+                    'fastapi.route.operation_id': 'null',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -410,7 +433,7 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
     assert exporter.exported_spans_as_dict(_include_pending_spans=True) == snapshot(
         [
             {
-                'name': 'outside request handler (pending)',
+                'name': 'outside request handler',
                 'context': {'trace_id': 1, 'span_id': 2, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'start_time': 1000000000,
@@ -426,7 +449,7 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                 },
             },
             {
-                'name': 'GET / (pending)',
+                'name': 'GET /',
                 'context': {'trace_id': 1, 'span_id': 4, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'start_time': 2000000000,
@@ -458,7 +481,7 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                 },
             },
             {
-                'name': 'FastAPI arguments (pending)',
+                'name': 'FastAPI arguments',
                 'context': {'trace_id': 1, 'span_id': 6, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 5, 'is_remote': False},
                 'start_time': 3000000000,
@@ -480,11 +503,16 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                     'logfire.span_type': 'span',
                     'logfire.msg_template': 'FastAPI arguments',
                     'logfire.msg': 'FastAPI arguments',
+                    'http.method': 'GET',
+                    'fastapi.route.name': 'homepage',
+                    'http.route': '/',
+                    'fastapi.route.operation_id': 'null',
                     'logfire.level_num': 5,
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                 },
             },
             {
-                'name': '{method} {http.route} ({code.function}) (pending)',
+                'name': '{method} {http.route} ({code.function})',
                 'context': {'trace_id': 1, 'span_id': 8, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 7, 'is_remote': False},
                 'start_time': 5000000000,
@@ -539,7 +567,7 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                 },
             },
             {
-                'name': 'GET / http send (pending)',
+                'name': 'GET / http send',
                 'context': {'trace_id': 1, 'span_id': 11, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 10, 'is_remote': False},
                 'start_time': 8000000000,
@@ -567,7 +595,7 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                 },
             },
             {
-                'name': 'GET / http send (pending)',
+                'name': 'GET / http send',
                 'context': {'trace_id': 1, 'span_id': 13, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 12, 'is_remote': False},
                 'start_time': 10000000000,
@@ -621,6 +649,9 @@ def test_fastapi_instrumentation(client: TestClient, exporter: TestExporter) ->
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/',
+                    'fastapi.route.name': 'homepage',
+                    'fastapi.route.operation_id': 'null',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -747,6 +778,12 @@ def test_fastapi_arguments(client: TestClient, exporter: TestExporter) -> None:
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/other',
+                    'fastapi.route.name': 'other_route_name',
+                    'fastapi.route.operation_id': 'other_route_operation_id',
+                    'fastapi.arguments.values': '{"foo":"foo_val"}',
+                    'fastapi.arguments.errors': '[{"type":"int_parsing","loc":["query","bar"],"msg":"Input should be a valid integer, unable to parse string as an integer","input":"bar_val"}]',
+                    'custom_attr': 'custom_value',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{},"custom_attr":{},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array","items":{"type":"object","properties":{"loc":{"type":"array","x-python-datatype":"tuple"}}}}}}',
                     'http.status_code': 422,
                     'http.response.status_code': 422,
                 },
@@ -872,6 +909,12 @@ def test_get_fastapi_arguments(client: TestClient, exporter: TestExporter) -> No
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/other',
+                    'fastapi.route.name': 'other_route_name',
+                    'fastapi.route.operation_id': 'other_route_operation_id',
+                    'fastapi.arguments.values': '{"foo":"foo_val","bar":1}',
+                    'fastapi.arguments.errors': '[]',
+                    'custom_attr': 'custom_value',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{},"custom_attr":{},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -997,6 +1040,12 @@ def test_first_lvl_subapp_fastapi_arguments(client: TestClient, exporter: TestEx
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/first_lvl',
+                    'fastapi.route.name': 'other_route_name',
+                    'fastapi.route.operation_id': 'other_route_operation_id',
+                    'fastapi.arguments.values': '{"foo":"foo_val","bar":1}',
+                    'fastapi.arguments.errors': '[]',
+                    'custom_attr': 'custom_value',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{},"custom_attr":{},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -1122,6 +1171,12 @@ def test_second_lvl_subapp_fastapi_arguments(client: TestClient, exporter: TestE
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/first_lvl',
+                    'fastapi.route.name': 'other_route_name',
+                    'fastapi.route.operation_id': 'other_route_operation_id',
+                    'fastapi.arguments.values': '{"foo":"foo_val","bar":1}',
+                    'fastapi.arguments.errors': '[]',
+                    'custom_attr': 'custom_value',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{},"custom_attr":{},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -1146,7 +1201,12 @@ def test_fastapi_unhandled_exception(client: TestClient, exporter: TestExporter)
                     'logfire.msg_template': 'FastAPI arguments',
                     'logfire.msg': 'FastAPI arguments',
                     'logfire.span_type': 'span',
+                    'http.method': 'GET',
+                    'fastapi.route.name': 'exception',
+                    'http.route': '/exception',
+                    'fastapi.route.operation_id': 'null',
                     'logfire.level_num': 5,
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                 },
             },
             {
@@ -1209,6 +1269,9 @@ def test_fastapi_unhandled_exception(client: TestClient, exporter: TestExporter)
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/exception',
+                    'fastapi.route.name': 'exception',
+                    'fastapi.route.operation_id': 'null',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                     'logfire.level_num': 17,
                 },
                 'events': [
@@ -1246,7 +1309,12 @@ def test_fastapi_handled_exception(client: TestClient, exporter: TestExporter) -
                     'logfire.msg_template': 'FastAPI arguments',
                     'logfire.msg': 'FastAPI arguments',
                     'logfire.span_type': 'span',
+                    'http.method': 'GET',
+                    'fastapi.route.name': 'validation_error',
+                    'http.route': '/validation_error',
+                    'fastapi.route.operation_id': 'null',
                     'logfire.level_num': 5,
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                 },
             },
             {
@@ -1337,6 +1405,9 @@ def test_fastapi_handled_exception(client: TestClient, exporter: TestExporter) -
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/validation_error',
+                    'fastapi.route.name': 'validation_error',
+                    'fastapi.route.operation_id': 'null',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"}}}',
                     'http.status_code': 422,
                     'http.response.status_code': 422,
                 },
@@ -1377,11 +1448,11 @@ def test_scrubbing(client: TestClient, exporter: TestExporter) -> None:
                     ),
                     'errors': '[]',
                     'custom_attr': 'custom_value',
+                    'fastapi.route.operation_id': 'null',
                     'http.method': 'GET',
                     'http.route': '/secret/{path_param}',
                     'fastapi.route.name': 'secret',
-                    'logfire.null_args': ('fastapi.route.operation_id',),
-                    'logfire.json_schema': '{"type":"object","properties":{"values":{"type":"object"},"errors":{"type":"array"},"custom_attr":{},"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{}}}',
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"values":{"type":"object"},"errors":{"type":"array"},"custom_attr":{}}}',
                     'logfire.scrubbed': IsJson(
                         [
                             {'path': ['attributes', 'values', 'path_param'], 'matched_substring': 'auth'},
@@ -1469,10 +1540,33 @@ def test_scrubbing(client: TestClient, exporter: TestExporter) -> None:
                     'client.port': 50000,
                     'http.route': '/secret/{path_param}',
                     'http.request.header.testauthorization': ("[Scrubbed due to 'auth']",),
+                    'fastapi.route.name': 'secret',
+                    'fastapi.route.operation_id': 'null',
+                    'fastapi.arguments.values': '{"path_param": "[Scrubbed due to \'auth\']", "foo": "foo_val", "password": "[Scrubbed due to \'password\']", "testauthorization": "[Scrubbed due to \'auth\']"}',
+                    'fastapi.arguments.errors': '[]',
+                    'custom_attr': 'custom_value',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"custom_attr":{},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                     'logfire.scrubbed': IsJson(
-                        [{'path': ['attributes', 'http.request.header.testauthorization'], 'matched_substring': 'auth'}]
+                        [
+                            {
+                                'path': ['attributes', 'http.request.header.testauthorization'],
+                                'matched_substring': 'auth',
+                            },
+                            {
+                                'path': ['attributes', 'fastapi.arguments.values', 'path_param'],
+                                'matched_substring': 'auth',
+                            },
+                            {
+                                'path': ['attributes', 'fastapi.arguments.values', 'password'],
+                                'matched_substring': 'password',
+                            },
+                            {
+                                'path': ['attributes', 'fastapi.arguments.values', 'testauthorization'],
+                                'matched_substring': 'auth',
+                            },
+                        ]
                     ),
                 },
             },
@@ -1490,10 +1584,14 @@ def make_request_hook_spans(record_send_receive: bool):
     app.post('/echo_body')(echo_body)
     client = TestClient(app)
 
+    def server_request_hook(span: Any, _scope: dict[str, Any]):
+        logfire.info('server_request_hook')
+        set_user_attributes_on_raw_span(span, {'attr_key': 'attr_val'})
+
     with logfire.instrument_fastapi(
         app,
         record_send_receive=record_send_receive,
-        server_request_hook=lambda *_, **__: logfire.info('server_request_hook'),  # type: ignore
+        server_request_hook=server_request_hook,
         client_request_hook=lambda *_, **__: logfire.info('client_request_hook'),  # type: ignore
         client_response_hook=lambda *_, **__: logfire.info('client_response_hook'),  # type: ignore
     ):
@@ -1518,7 +1616,7 @@ def test_request_hooks_without_send_receiev_spans(exporter: TestExporter):
                     'logfire.msg_template': 'server_request_hook',
                     'logfire.msg': 'server_request_hook',
                     'code.filepath': 'test_fastapi.py',
-                    'code.function': '<lambda>',
+                    'code.function': 'server_request_hook',
                     'code.lineno': 123,
                 },
             },
@@ -1535,10 +1633,10 @@ def test_request_hooks_without_send_receiev_spans(exporter: TestExporter):
                     'values': '{}',
                     'errors': '[]',
                     'http.method': 'POST',
+                    'fastapi.route.operation_id': 'null',
                     'http.route': '/echo_body',
                     'fastapi.route.name': 'echo_body',
-                    'logfire.null_args': ('fastapi.route.operation_id',),
-                    'logfire.json_schema': '{"type":"object","properties":{"values":{"type":"object"},"errors":{"type":"array"},"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{}}}',
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"values":{"type":"object"},"errors":{"type":"array"}}}',
                 },
             },
             {
@@ -1637,6 +1735,12 @@ def test_request_hooks_without_send_receiev_spans(exporter: TestExporter):
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/echo_body',
+                    'attr_key': 'attr_val',
+                    'fastapi.route.name': 'echo_body',
+                    'fastapi.route.operation_id': 'null',
+                    'fastapi.arguments.values': '{}',
+                    'fastapi.arguments.errors': '[]',
+                    'logfire.json_schema': '{"type":"object","properties":{"attr_key":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -1661,7 +1765,7 @@ def test_request_hooks_with_send_receive_spans(exporter: TestExporter):
                     'logfire.msg_template': 'server_request_hook',
                     'logfire.msg': 'server_request_hook',
                     'code.filepath': 'test_fastapi.py',
-                    'code.function': '<lambda>',
+                    'code.function': 'server_request_hook',
                     'code.lineno': 123,
                 },
             },
@@ -1678,10 +1782,10 @@ def test_request_hooks_with_send_receive_spans(exporter: TestExporter):
                     'values': '{}',
                     'errors': '[]',
                     'http.method': 'POST',
+                    'fastapi.route.operation_id': 'null',
                     'http.route': '/echo_body',
                     'fastapi.route.name': 'echo_body',
-                    'logfire.null_args': ('fastapi.route.operation_id',),
-                    'logfire.json_schema': '{"type":"object","properties":{"values":{"type":"object"},"errors":{"type":"array"},"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{}}}',
+                    'logfire.json_schema': '{"type":"object","properties":{"http.method":{},"http.route":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"values":{"type":"object"},"errors":{"type":"array"}}}',
                 },
             },
             {
@@ -1821,6 +1925,12 @@ def test_request_hooks_with_send_receive_spans(exporter: TestExporter):
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/echo_body',
+                    'attr_key': 'attr_val',
+                    'fastapi.route.name': 'echo_body',
+                    'fastapi.route.operation_id': 'null',
+                    'fastapi.arguments.values': '{}',
+                    'fastapi.arguments.errors': '[]',
+                    'logfire.json_schema': '{"type":"object","properties":{"attr_key":{},"fastapi.route.name":{},"fastapi.route.operation_id":{"type":"null"},"fastapi.arguments.values":{"type":"object"},"fastapi.arguments.errors":{"type":"array"}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
@@ -1847,7 +1957,10 @@ def test_websocket(client: TestClient, exporter: TestExporter) -> None:
                     'logfire.msg_template': 'FastAPI arguments',
                     'logfire.msg': 'FastAPI arguments',
                     'logfire.span_type': 'span',
+                    'fastapi.route.name': 'websocket_endpoint',
+                    'http.route': '/ws/{name}',
                     'logfire.level_num': 5,
+                    'logfire.json_schema': '{"type":"object","properties":{"http.route":{},"fastapi.route.name":{}}}',
                 },
             },
             {
@@ -1962,9 +2075,19 @@ def test_websocket(client: TestClient, exporter: TestExporter) -> None:
                     'net.peer.port': 50000,
                     'client.port': 50000,
                     'http.route': '/ws/{name}',
+                    'fastapi.route.name': 'websocket_endpoint',
+                    'logfire.json_schema': '{"type":"object","properties":{"fastapi.route.name":{}}}',
                     'http.status_code': 200,
                     'http.response.status_code': 200,
                 },
             },
         ]
     )
+
+
+def test_sampled_out(client: TestClient, exporter: TestExporter, config_kwargs: dict[str, Any]) -> None:
+    logfire.configure(**config_kwargs, sampling=logfire.SamplingOptions(head=0))
+    make_request_hook_spans(record_send_receive=True)
+    make_request_hook_spans(record_send_receive=False)
+
+    assert exporter.exported_spans_as_dict() == []
