@@ -1,5 +1,6 @@
 import ast
 import asyncio
+import runpy
 import sys
 from importlib.machinery import SourceFileLoader
 from typing import Any, Callable, ContextManager
@@ -51,12 +52,15 @@ def test_auto_trace_sample(exporter: TestExporter) -> None:
     with pytest.raises(IndexError):  # foo.bar intentionally raises an error to test that it's recorded below
         asyncio.run(foo.bar())
 
+    # Simulate `python -m tests.auto_trace_samples`
+    runpy.run_module('tests.auto_trace_samples')
+
     assert exporter.exported_spans[0].instrumentation_scope.name == 'logfire.auto_tracing'  # type: ignore
 
-    assert exporter.exported_spans_as_dict(_include_pending_spans=True) == snapshot(
+    assert exporter.exported_spans_as_dict(_include_pending_spans=True, _strip_function_qualname=False) == snapshot(
         [
             {
-                'name': 'Calling tests.auto_trace_samples.foo.bar (pending)',
+                'name': 'Calling tests.auto_trace_samples.foo.bar',
                 'context': {'trace_id': 1, 'span_id': 2, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'start_time': 1000000000,
@@ -73,7 +77,7 @@ def test_auto_trace_sample(exporter: TestExporter) -> None:
                 },
             },
             {
-                'name': 'Calling async_gen via @instrument (pending)',
+                'name': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
                 'context': {'trace_id': 1, 'span_id': 4, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'start_time': 2000000000,
@@ -81,15 +85,16 @@ def test_auto_trace_sample(exporter: TestExporter) -> None:
                 'attributes': {
                     'code.filepath': 'foo.py',
                     'code.lineno': 123,
-                    'code.function': 'async_gen',
-                    'logfire.msg_template': 'Calling async_gen via @instrument',
+                    'code.function': 'async_gen.<locals>.inner',
+                    'logfire.msg_template': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
+                    'logfire.tags': ('testing', 'auto-tracing'),
                     'logfire.span_type': 'pending_span',
-                    'logfire.msg': 'Calling async_gen via @instrument',
+                    'logfire.msg': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
                     'logfire.pending_parent_id': '0000000000000001',
                 },
             },
             {
-                'name': 'Calling async_gen via @instrument',
+                'name': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
                 'context': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'start_time': 2000000000,
@@ -97,10 +102,11 @@ def test_auto_trace_sample(exporter: TestExporter) -> None:
                 'attributes': {
                     'code.filepath': 'foo.py',
                     'code.lineno': 123,
-                    'code.function': 'async_gen',
-                    'logfire.msg_template': 'Calling async_gen via @instrument',
+                    'code.function': 'async_gen.<locals>.inner',
+                    'logfire.msg_template': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
+                    'logfire.tags': ('testing', 'auto-tracing'),
                     'logfire.span_type': 'span',
-                    'logfire.msg': 'Calling async_gen via @instrument',
+                    'logfire.msg': 'Calling tests.auto_trace_samples.foo.async_gen.<locals>.inner',
                 },
             },
             {
@@ -132,6 +138,37 @@ def test_auto_trace_sample(exporter: TestExporter) -> None:
                     }
                 ],
             },
+            {
+                'name': 'Calling tests.auto_trace_samples.__main__.main',
+                'context': {'trace_id': 2, 'span_id': 6, 'is_remote': False},
+                'parent': {'trace_id': 2, 'span_id': 5, 'is_remote': False},
+                'start_time': 6000000000,
+                'end_time': 6000000000,
+                'attributes': {
+                    'code.filepath': '__main__.py',
+                    'code.lineno': 123,
+                    'code.function': 'main',
+                    'logfire.msg_template': 'Calling tests.auto_trace_samples.__main__.main',
+                    'logfire.span_type': 'pending_span',
+                    'logfire.msg': 'Calling tests.auto_trace_samples.__main__.main',
+                    'logfire.pending_parent_id': '0000000000000000',
+                },
+            },
+            {
+                'name': 'Calling tests.auto_trace_samples.__main__.main',
+                'context': {'trace_id': 2, 'span_id': 5, 'is_remote': False},
+                'parent': None,
+                'start_time': 6000000000,
+                'end_time': 7000000000,
+                'attributes': {
+                    'code.filepath': '__main__.py',
+                    'code.lineno': 123,
+                    'code.function': 'main',
+                    'logfire.msg_template': 'Calling tests.auto_trace_samples.__main__.main',
+                    'logfire.span_type': 'span',
+                    'logfire.msg': 'Calling tests.auto_trace_samples.__main__.main',
+                },
+            },
         ]
     )
 
@@ -163,8 +200,10 @@ def test_check_already_imported() -> None:
 
 
 # language=Python
-nested_sample = """
+nested_sample = '''
 def func():
+    """A docstring"""
+
     x = 1
 
     class Class:
@@ -191,13 +230,23 @@ class Class3:
     def method4(self):
         b = 7
         return b
-"""
+
+def only_docstring_function():
+    """Empty body"""
+
+def only_pass_function():
+    """Trivial body"""
+    pass
+
+def only_ellipsis_function():
+    ...
+'''
 
 
 def test_rewrite_ast():
     context_factories: list[Callable[[], ContextManager[Any]]] = []
     tree = rewrite_ast(
-        nested_sample,
+        ast.parse(nested_sample),
         'foo.py',
         'logfire_span',
         'module.name',
@@ -205,8 +254,9 @@ def test_rewrite_ast():
         context_factories,
         min_duration=0,
     )
-    result = """
+    result = '''
 def func():
+    """A docstring"""
     with logfire_span[3]():
         x = 1
 
@@ -238,7 +288,17 @@ class Class3:
         with logfire_span[4]():
             b = 7
             return b
-"""
+
+def only_docstring_function():
+    """Empty body"""
+
+def only_pass_function():
+    """Trivial body"""
+    pass
+
+def only_ellipsis_function():
+    ...
+'''
 
     if sys.version_info >= (3, 9):  # pragma: no branch
         assert ast.unparse(tree).strip() == result.strip()
@@ -256,7 +316,7 @@ class Class3:
                 'Calling module.name.func.<locals>.Class.method',
                 {
                     'code.filepath': 'foo.py',
-                    'code.lineno': 8,
+                    'code.lineno': 10,
                     'code.function': 'func.<locals>.Class.method',
                     'logfire.msg_template': 'Calling module.name.func.<locals>.Class.method',
                 },
@@ -265,7 +325,7 @@ class Class3:
                 'Calling module.name.func.<locals>.Class.method2.<locals>.Class2.method3',
                 {
                     'code.filepath': 'foo.py',
-                    'code.lineno': 16,
+                    'code.lineno': 18,
                     'code.function': 'func.<locals>.Class.method2.<locals>.Class2.method3',
                     'logfire.msg_template': 'Calling module.name.func.<locals>.Class.method2.<locals>.Class2.method3',
                 },
@@ -274,7 +334,7 @@ class Class3:
                 'Calling module.name.func.<locals>.Class.method2',
                 {
                     'code.filepath': 'foo.py',
-                    'code.lineno': 12,
+                    'code.lineno': 14,
                     'code.function': 'func.<locals>.Class.method2',
                     'logfire.msg_template': 'Calling module.name.func.<locals>.Class.method2',
                 },
@@ -292,7 +352,7 @@ class Class3:
                 'Calling module.name.Class3.method4',
                 {
                     'code.filepath': 'foo.py',
-                    'code.lineno': 26,
+                    'code.lineno': 28,
                     'code.function': 'Class3.method4',
                     'logfire.msg_template': 'Calling module.name.Class3.method4',
                 },
@@ -331,7 +391,7 @@ from logfire import no_auto_trace
 @str
 def traced_func():
     async def inner():
-        pass
+        return 1
     return inner
 
 
@@ -340,18 +400,18 @@ def traced_func():
 @str
 def not_traced_func():
     async def inner():
-        pass
+        return 1
     return inner
 
 
 @str
 class TracedClass:
     async def traced_method(self):
-        pass
+        return 1
 
     @no_auto_trace
     def not_traced_method(self):
-        pass
+        return 1
 
 
 @no_auto_trace
@@ -359,19 +419,19 @@ class TracedClass:
 class NotTracedClass:
     async def would_be_traced_method(self):
         def inner():
-            pass
+            return 1
         return inner
 
     @no_auto_trace
     def definitely_not_traced_method(self):
-        pass
+        return 1
 """
 
 
 def get_calling_strings(sample: str):
     context_factories: list[Callable[[], ContextManager[Any]]] = []
     rewrite_ast(
-        sample,
+        ast.parse(sample),
         'foo.py',
         'logfire_span',
         'module.name',
@@ -417,13 +477,12 @@ def test_no_auto_trace():
     )
 
 
-# language=Python
 generators_sample = """
 def make_gen():
     def gen():
         async def foo():
             async def bar():
-                pass
+                return lambda: (yield 1)
             yield bar()
         yield from foo()
     return gen
