@@ -5,20 +5,21 @@ import re
 import sys
 from collections import deque
 from collections.abc import Sequence
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from datetime import date, datetime, time, timedelta
 from decimal import Decimal
 from enum import Enum
 from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
 from pathlib import Path
 from typing import Any, Iterator, List, Mapping
+from unittest.mock import MagicMock, Mock
 from uuid import UUID
 
 import numpy
 import pandas
 import pytest
 from attrs import define
-from dirty_equals import IsJson
+from dirty_equals import IsJson, IsStr
 from inline_snapshot import snapshot
 from pydantic import AnyUrl, BaseModel, ConfigDict, FilePath, NameEmail, SecretBytes, SecretStr
 from pydantic.dataclasses import dataclass as pydantic_dataclass
@@ -71,6 +72,12 @@ class MyPydanticComplexDataclass:
     t: MyPydanticDataclass
 
 
+@dataclass
+class MyReprDataclass:
+    in_repr: int
+    not_in_repr: MyDataclass = field(repr=False)
+
+
 class MySQLModel(SQLModel):
     s: int
 
@@ -156,6 +163,9 @@ class StrSubclass(str):
     pass
 
 
+ANYURL_REPR_CLASSNAME = repr(AnyUrl('http://test.com')).split('(')[0]
+
+
 @pytest.mark.parametrize(
     'value,value_repr,value_json,json_schema',
     [
@@ -489,26 +499,26 @@ class StrSubclass(str):
         ),
         pytest.param(
             MyModel(x='x', y=10, u=AnyUrl('http://test.com')),
-            "x='x' y=10 u=Url('http://test.com/')",
+            f"x='x' y=10 u={ANYURL_REPR_CLASSNAME}('http://test.com/')",
             '{"x":"x","y":10,"u":"http://test.com/"}',
             {
                 'type': 'object',
                 'title': 'MyModel',
                 'x-python-datatype': 'PydanticModel',
-                'properties': {'u': {'type': 'string', 'x-python-datatype': 'Url'}},
+                'properties': {'u': {'type': 'string', 'x-python-datatype': ANYURL_REPR_CLASSNAME}},
             },
             id='pydantic_model',
         ),
         pytest.param(
             MyModel.model_validate(dict(x='x', y=10, u='http://test.com', extra_key=MyDataclass(10))),
-            "x='x' y=10 u=Url('http://test.com/')",
+            f"x='x' y=10 u={ANYURL_REPR_CLASSNAME}('http://test.com/')",
             '{"x":"x","y":10,"u":"http://test.com/","extra_key":{"t":10}}',
             {
                 'type': 'object',
                 'title': 'MyModel',
                 'x-python-datatype': 'PydanticModel',
                 'properties': {
-                    'u': {'type': 'string', 'x-python-datatype': 'Url'},
+                    'u': {'type': 'string', 'x-python-datatype': ANYURL_REPR_CLASSNAME},
                     'extra_key': {
                         'type': 'object',
                         'title': 'MyDataclass',
@@ -574,6 +584,13 @@ class StrSubclass(str):
             },
             id='pydantic_complex_dataclass',
         ),
+        pytest.param(
+            MyReprDataclass(in_repr=1, not_in_repr=MyDataclass(t=2)),
+            'MyReprDataclass(in_repr=1)',
+            '{"in_repr":1}',
+            {'type': 'object', 'title': 'MyReprDataclass', 'x-python-datatype': 'dataclass'},
+            id='repr_dataclass',
+        ),
         pytest.param(
             ValueError('Test value error'),
             'Test value error',
@@ -807,7 +824,7 @@ class StrSubclass(str):
         pytest.param(
             [{str: bytes, int: float}],
             "[{<class 'str'>: <class 'bytes'>, <class 'int'>: <class 'float'>}]",
-            '[{"<class \'str\'>":"<class \'bytes\'>","<class \'int\'>":"<class ' "'float'>\"}]",
+            '[{"<class \'str\'>":"<class \'bytes\'>","<class \'int\'>":"<class \'float\'>"}]',
             {
                 'items': {
                     'properties': {
@@ -887,6 +904,7 @@ def test_log_sqlalchemy_class(exporter: TestExporter) -> None:
     var = session.query(SAModel).all()[0]
     var2 = session.query(SAModel2).all()[0]
     logfire.info('test message', var=var, var2=var2)
+    engine.dispose()
 
     assert exporter.exported_spans_as_dict() == snapshot(
         [
@@ -1184,3 +1202,223 @@ def test_recursive_objects(exporter: TestExporter) -> None:
             }
         ]
     )
+
+
+def test_repeated_objects(exporter: TestExporter) -> None:
+    @dataclass
+    class Model:
+        x: Any
+
+    x = Model(x=1)
+    m = Model(x=[x, x])
+
+    logfire.info('hi', m=m)
+
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'hi',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 1000000000,
+                'attributes': {
+                    'logfire.span_type': 'log',
+                    'logfire.level_num': 9,
+                    'logfire.msg_template': 'hi',
+                    'logfire.msg': 'hi',
+                    'code.filepath': 'test_json_args.py',
+                    'code.function': 'test_repeated_objects',
+                    'code.lineno': 123,
+                    'm': '{"x":[{"x":1},{"x":1}]}',
+                    'logfire.json_schema': '{"type":"object","properties":{"m":{"type":"object","title":"Model","x-python-datatype":"dataclass","properties":{"x":{"type":"array","items":{"type":"object","title":"Model","x-python-datatype":"dataclass"}}}}}}',
+                },
+            }
+        ]
+    )
+
+
+def test_numpy_array_truncation(exporter: TestExporter):
+    logfire.info('hi', m=numpy.arange(13 * 3 * 11).reshape(13, 3, 11))
+
+    truncated = [
+        [
+            [0, 1, 2, 3, 4, 6, 7, 8, 9, 10],
+            [11, 12, 13, 14, 15, 17, 18, 19, 20, 21],
+            [22, 23, 24, 25, 26, 28, 29, 30, 31, 32],
+        ],
+        [
+            [33, 34, 35, 36, 37, 39, 40, 41, 42, 43],
+            [44, 45, 46, 47, 48, 50, 51, 52, 53, 54],
+            [55, 56, 57, 58, 59, 61, 62, 63, 64, 65],
+        ],
+        [
+            [66, 67, 68, 69, 70, 72, 73, 74, 75, 76],
+            [77, 78, 79, 80, 81, 83, 84, 85, 86, 87],
+            [88, 89, 90, 91, 92, 94, 95, 96, 97, 98],
+        ],
+        [
+            [99, 100, 101, 102, 103, 105, 106, 107, 108, 109],
+            [110, 111, 112, 113, 114, 116, 117, 118, 119, 120],
+            [121, 122, 123, 124, 125, 127, 128, 129, 130, 131],
+        ],
+        [
+            [132, 133, 134, 135, 136, 138, 139, 140, 141, 142],
+            [143, 144, 145, 146, 147, 149, 150, 151, 152, 153],
+            [154, 155, 156, 157, 158, 160, 161, 162, 163, 164],
+        ],
+        [
+            [264, 265, 266, 267, 268, 270, 271, 272, 273, 274],
+            [275, 276, 277, 278, 279, 281, 282, 283, 284, 285],
+            [286, 287, 288, 289, 290, 292, 293, 294, 295, 296],
+        ],
+        [
+            [297, 298, 299, 300, 301, 303, 304, 305, 306, 307],
+            [308, 309, 310, 311, 312, 314, 315, 316, 317, 318],
+            [319, 320, 321, 322, 323, 325, 326, 327, 328, 329],
+        ],
+        [
+            [330, 331, 332, 333, 334, 336, 337, 338, 339, 340],
+            [341, 342, 343, 344, 345, 347, 348, 349, 350, 351],
+            [352, 353, 354, 355, 356, 358, 359, 360, 361, 362],
+        ],
+        [
+            [363, 364, 365, 366, 367, 369, 370, 371, 372, 373],
+            [374, 375, 376, 377, 378, 380, 381, 382, 383, 384],
+            [385, 386, 387, 388, 389, 391, 392, 393, 394, 395],
+        ],
+        [
+            [396, 397, 398, 399, 400, 402, 403, 404, 405, 406],
+            [407, 408, 409, 410, 411, 413, 414, 415, 416, 417],
+            [418, 419, 420, 421, 422, 424, 425, 426, 427, 428],
+        ],
+    ]
+    assert numpy.array(truncated).shape == (10, 3, 10)
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'hi',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 1000000000,
+                'attributes': {
+                    'logfire.span_type': 'log',
+                    'logfire.level_num': 9,
+                    'logfire.msg_template': 'hi',
+                    'logfire.msg': 'hi',
+                    'code.filepath': 'test_json_args.py',
+                    'code.function': 'test_numpy_array_truncation',
+                    'code.lineno': 123,
+                    'm': IsJson(truncated),
+                    'logfire.json_schema': '{"type":"object","properties":{"m":{"type":"array","x-python-datatype":"ndarray","x-shape":[13,3,11],"x-dtype":"int64"}}}',
+                },
+            }
+        ]
+    )
+
+
+def test_bad_getattr(exporter: TestExporter, caplog: pytest.LogCaptureFixture):
+    class A:
+        def __getattr__(self, item: str):
+            raise RuntimeError
+
+        def __repr__(self):
+            return 'A()'
+
+    logfire.info('hello', a=A())
+
+    assert not caplog.messages
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'hello',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 1000000000,
+                'attributes': {
+                    'logfire.span_type': 'log',
+                    'logfire.level_num': 9,
+                    'logfire.msg_template': 'hello',
+                    'logfire.msg': 'hello',
+                    'code.filepath': 'test_json_args.py',
+                    'code.function': 'test_bad_getattr',
+                    'code.lineno': 123,
+                    'a': '"A()"',
+                    'logfire.json_schema': '{"type":"object","properties":{"a":{"type":"object","x-python-datatype":"unknown"}}}',
+                },
+            }
+        ]
+    )
+
+
+def test_to_dict(exporter: TestExporter):
+    class Foo:
+        def to_dict(self):
+            return {'x': 1}
+
+    logfire.info('hi', foo=Foo())
+
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'hi',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 1000000000,
+                'attributes': {
+                    'logfire.span_type': 'log',
+                    'logfire.level_num': 9,
+                    'logfire.msg_template': 'hi',
+                    'logfire.msg': 'hi',
+                    'code.filepath': 'test_json_args.py',
+                    'code.function': 'test_to_dict',
+                    'code.lineno': 123,
+                    'foo': '{"x":1}',
+                    'logfire.json_schema': '{"type":"object","properties":{"foo":{"type":"object","x-python-datatype":"unknown"}}}',
+                },
+            }
+        ]
+    )
+
+
+def test_mock(exporter: TestExporter):
+    class Mixin:
+        def __repr__(self):
+            return f'{self.__class__.__name__}()'
+
+    class Foo(Mixin, Mock):
+        pass
+
+    class Bar(Mixin, MagicMock):
+        pass
+
+    logfire.info('hi', foo=Foo(), bar=Bar(), mock=Mock(), magic_mock=MagicMock())
+
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'hi',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 1000000000,
+                'attributes': {
+                    'logfire.span_type': 'log',
+                    'logfire.level_num': 9,
+                    'logfire.msg_template': 'hi',
+                    'logfire.msg': 'hi',
+                    'code.filepath': 'test_json_args.py',
+                    'code.function': 'test_mock',
+                    'code.lineno': 123,
+                    'foo': '"Foo()"',
+                    'bar': '"Bar()"',
+                    'mock': IsStr(regex=r'^"<Mock id=\'\d+\'>"'),
+                    'magic_mock': IsStr(regex=r'^"<MagicMock id=\'\d+\'>"'),
+                    'logfire.json_schema': '{"type":"object","properties":{"foo":{"type":"object","x-python-datatype":"unknown"},"bar":{"type":"object","x-python-datatype":"unknown"},"mock":{"type":"object","x-python-datatype":"unknown"},"magic_mock":{"type":"object","x-python-datatype":"unknown"}}}',
+                },
+            }
+        ]
+    )
