@@ -2,26 +2,33 @@ from __future__ import annotations
 
 import importlib.metadata
 import os
-from typing import Any
+from typing import TYPE_CHECKING, Any
 from unittest.mock import patch
 
 import cloudpickle
 import pytest
+import sqlmodel
 from dirty_equals import IsInt
 from inline_snapshot import snapshot
 from opentelemetry.sdk.metrics.export import AggregationTemporality, InMemoryMetricReader
 from opentelemetry.sdk.trace.export import SimpleSpanProcessor
-from pydantic import BaseModel, ConfigDict, ValidationError, field_validator
+from pydantic import (
+    AfterValidator,
+    BaseModel,
+    ConfigDict,
+    TypeAdapter,
+    ValidationError,
+    __version__ as pydantic_version,
+    field_validator,
+)
 from pydantic.dataclasses import dataclass as pydantic_dataclass
-from pydantic.functional_validators import AfterValidator
-from pydantic.plugin import PydanticPluginProtocol, SchemaTypePath, ValidatePythonHandlerProtocol
-from pydantic.type_adapter import TypeAdapter
 from pydantic_core import core_schema
 from typing_extensions import Annotated
 
 import logfire
 import logfire.integrations.pydantic
 from logfire._internal.config import GLOBAL_CONFIG
+from logfire._internal.utils import get_version
 from logfire.integrations.pydantic import (
     LogfirePydanticPlugin,
     get_schema_name,
@@ -29,6 +36,20 @@ from logfire.integrations.pydantic import (
 from logfire.testing import SeededRandomIdGenerator, TestExporter
 from tests.test_metrics import get_collected_metrics
 
+pytestmark = pytest.mark.skipif(
+    get_version(pydantic_version) < get_version('2.5.0'),
+    reason='Skipping all tests for versions less than 2.5.',
+)
+
+if TYPE_CHECKING:
+    from pydantic.plugin import PydanticPluginProtocol, SchemaTypePath, ValidatePythonHandlerProtocol
+
+try:
+    from pydantic.plugin import PydanticPluginProtocol, SchemaTypePath, ValidatePythonHandlerProtocol
+except ImportError:
+    # it's fine, pydantic version <v2.5
+    pass
+
 
 def test_plugin_listed():
     found = True
@@ -202,6 +223,7 @@ def test_pydantic_plugin_python_record_failure(exporter: TestExporter, metrics_r
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 1,
+                            'exemplars': [],
                         },
                         {
                             'attributes': {
@@ -212,6 +234,7 @@ def test_pydantic_plugin_python_record_failure(exporter: TestExporter, metrics_r
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 1,
+                            'exemplars': [],
                         },
                     ],
                     'aggregation_temporality': AggregationTemporality.DELTA,
@@ -254,6 +277,7 @@ def test_pydantic_plugin_metrics(metrics_reader: InMemoryMetricReader) -> None:
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 3,
+                            'exemplars': [],
                         },
                         {
                             'attributes': {
@@ -264,6 +288,7 @@ def test_pydantic_plugin_metrics(metrics_reader: InMemoryMetricReader) -> None:
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 2,
+                            'exemplars': [],
                         },
                     ],
                     'aggregation_temporality': AggregationTemporality.DELTA,
@@ -325,6 +350,15 @@ def test_pydantic_plugin_python_success(exporter: TestExporter, metrics_reader:
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 1,
+                            'exemplars': [
+                                {
+                                    'filtered_attributes': {},
+                                    'value': 1,
+                                    'time_unix_nano': IsInt(),
+                                    'span_id': 1,
+                                    'trace_id': 1,
+                                }
+                            ],
                         }
                     ],
                     'aggregation_temporality': AggregationTemporality.DELTA,
@@ -410,6 +444,7 @@ def test_pydantic_plugin_python_error_record_failure(
                             'start_time_unix_nano': IsInt(gt=0),
                             'time_unix_nano': IsInt(gt=0),
                             'value': 2,
+                            'exemplars': [],
                         }
                     ],
                     'aggregation_temporality': AggregationTemporality.DELTA,
@@ -917,7 +952,7 @@ def test_pydantic_plugin_python_exception_record_failure(exporter: TestExporter)
 
 
 def test_old_plugin_style(exporter: TestExporter) -> None:
-    # Test that plguins for the old API still work together with the logfire plugin.
+    # Test that plugins for the old API still work together with the logfire plugin.
 
     events: list[str] = []
 
@@ -1215,6 +1250,7 @@ def test_record_metrics_env_var(metrics_reader: InMemoryMetricReader) -> None:
                                 'start_time_unix_nano': IsInt(gt=0),
                                 'time_unix_nano': IsInt(gt=0),
                                 'value': 1,
+                                'exemplars': [],
                             }
                         ],
                         'aggregation_temporality': 1,
@@ -1231,3 +1267,39 @@ def test_cloudpickle():
 
     m = MyModel(x=1)
     assert cloudpickle.loads(cloudpickle.dumps(m)).model_dump() == m.model_dump() == {'x': 1}  # type: ignore
+
+
+def test_sqlmodel_pydantic_plugin(exporter: TestExporter) -> None:
+    logfire.instrument_pydantic()
+
+    class Hero(sqlmodel.SQLModel, table=True):
+        id: int = sqlmodel.Field(default=1, primary_key=True)
+
+    Hero.model_validate({})
+
+    assert exporter.exported_spans_as_dict() == snapshot(
+        [
+            {
+                'name': 'pydantic.validate_python',
+                'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
+                'parent': None,
+                'start_time': 1000000000,
+                'end_time': 2000000000,
+                'attributes': {
+                    'code.filepath': 'test_pydantic_plugin.py',
+                    'code.function': 'test_sqlmodel_pydantic_plugin',
+                    'code.lineno': 123,
+                    'schema_name': 'Hero',
+                    'validation_method': 'validate_python',
+                    'input_data': '{}',
+                    'logfire.msg_template': 'Pydantic {schema_name} {validation_method}',
+                    'logfire.level_num': 9,
+                    'logfire.span_type': 'span',
+                    'success': True,
+                    'result': '{"id":1}',
+                    'logfire.msg': 'Pydantic Hero validate_python succeeded',
+                    'logfire.json_schema': '{"type":"object","properties":{"schema_name":{},"validation_method":{},"input_data":{"type":"object"},"success":{},"result":{"type":"object","title":"Hero","x-python-datatype":"PydanticModel"}}}',
+                },
+            }
+        ]
+    )
